pub struct LRLexError { /* private fields */ }Expand description
A Lexing error.
Implementations
Sourceimpl LRLexError
impl LRLexError
Sourcepub fn new_with_lexing_state(span: Span, lexing_state: StartStateId) -> Self
pub fn new_with_lexing_state(span: Span, lexing_state: StartStateId) -> Self
Construct a new LRLex error covering span for lexing_state.
Sourcepub fn lexing_state(&self) -> Option<StartStateId>
pub fn lexing_state(&self) -> Option<StartStateId>
Returns the state, if there was one, that the lexer was in when the error was detected.